@@ -2,7 +2,7 @@
 #
 # The MIT License (MIT)
 #
-# Copyright (c) Tavendo GmbH
+# Copyright (c) Crossbar.io Technologies GmbH
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -27,16 +27,19 @@
 import abc
 import six
 
+from autobahn.util import public
+
 __all__ = (
     'IObjectSerializer',
     'ISerializer',
+    'IMessage',
     'ITransport',
     'ITransportHandler',
     'ISession',
-    'IApplicationSession',
 )
 
 
+@public
 @six.add_metaclass(abc.ABCMeta)
 class IObjectSerializer(object):
     """
@@ -45,80 +48,145 @@ class IObjectSerializer(object):
     :class:`autobahn.wamp.interfaces.ISerializer`.
     """
 
+    @public
     @abc.abstractproperty
-    def BINARY(self):
+    def BINARY():
         """
         Flag (read-only) to indicate if serializer requires a binary clean
         transport or if UTF8 transparency is sufficient.
         """
 
+    @public
     @abc.abstractmethod
-    def serialize(self, obj):
+    def serialize(obj):
         """
         Serialize an object to a byte string.
 
         :param obj: Object to serialize.
-        :type obj: Any serializable type.
+        :type obj: any (serializable type)
 
-        :returns: bytes -- Serialized byte string.
+        :returns: Serialized bytes.
+        :rtype: bytes
         """
 
+    @public
     @abc.abstractmethod
-    def unserialize(self, payload):
+    def unserialize(payload):
         """
         Unserialize objects from a byte string.
 
         :param payload: Objects to unserialize.
         :type payload: bytes
 
-        :returns: list -- List of (raw) objects unserialized.
+        :returns: List of (raw) objects unserialized.
+        :rtype: list
         """
 
 
+@public
 @six.add_metaclass(abc.ABCMeta)
 class ISerializer(object):
     """
     WAMP message serialization and deserialization.
     """
 
+    @public
     @abc.abstractproperty
-    def MESSAGE_TYPE_MAP(self):
+    def MESSAGE_TYPE_MAP():
         """
         Mapping of WAMP message type codes to WAMP message classes.
         """
 
+    @public
     @abc.abstractproperty
-    def SERIALIZER_ID(self):
+    def SERIALIZER_ID():
         """
         The WAMP serialization format ID.
         """
 
+    @public
     @abc.abstractmethod
-    def serialize(self, message):
+    def serialize(message):
         """
         Serializes a WAMP message to bytes for sending over a transport.
 
-        :param message: An instance that implements :class:`autobahn.wamp.interfaces.IMessage`
-        :type message: obj
+        :param message: The WAMP message to be serialized.
+        :type message: object implementing :class:`autobahn.wamp.interfaces.IMessage`
 
-        :returns: tuple -- A pair ``(payload, is_binary)``.
+        :returns: A pair ``(payload, isBinary)``.
+        :rtype: tuple
         """
 
+    @public
     @abc.abstractmethod
-    def unserialize(self, payload, is_binary):
+    def unserialize(payload, isBinary):
         """
         Deserialize bytes from a transport and parse into WAMP messages.
 
         :param payload: Byte string from wire.
         :type payload: bytes
+
         :param is_binary: Type of payload. True if payload is a binary string, else
             the payload is UTF-8 encoded Unicode text.
         :type is_binary: bool
 
-        :returns: list -- List of ``a.w.m.Message`` objects.
+        :returns: List of ``a.w.m.Message`` objects.
+        :rtype: list
         """
 
 
+@public
+@six.add_metaclass(abc.ABCMeta)
+class IMessage(object):
+    """
+    """
+
+    @public
+    @abc.abstractproperty
+    def MESSAGE_TYPE():
+        """
+        WAMP message type code.
+        """
+
+    # the following requires Python 3.3+ and exactly this order of decorators
+    # http://stackoverflow.com/questions/4474395/staticmethod-and-abc-abstractmethod-will-it-blend
+    @public
+    @staticmethod
+    @abc.abstractmethod
+    def parse(wmsg):
+        """
+        Factory method that parses a unserialized raw message (as returned byte
+        :func:`autobahn.interfaces.ISerializer.unserialize`) into an instance
+        of this class.
+
+        :returns: The parsed WAMP message.
+        :rtype: object implementing :class:`autobahn.wamp.interfaces.IMessage`
+        """
+
+    @public
+    @abc.abstractmethod
+    def serialize(serializer):
+        """
+        Serialize this object into a wire level bytes representation and cache
+        the resulting bytes. If the cache already contains an entry for the given
+        serializer, return the cached representation directly.
+
+        :param serializer: The wire level serializer to use.
+        :type serializer: object implementing :class:`autobahn.wamp.interfaces.ISerializer`
+
+        :returns: The serialized bytes.
+        :rtype: bytes
+        """
+
+    @public
+    @abc.abstractmethod
+    def uncache():
+        """
+        Resets the serialization cache for this message.
+        """
+
+
+@public
 @six.add_metaclass(abc.ABCMeta)
 class ITransport(object):
     """
@@ -126,8 +194,9 @@ class ITransport(object):
     message-based channel.
     """
 
+    @public
     @abc.abstractmethod
-    def send(self, message):
+    def send(message):
         """
         Send a WAMP message over the transport to the peer. If the transport is
         not open, this raises :class:`autobahn.wamp.exception.TransportLost`.
@@ -135,30 +204,34 @@ class ITransport(object):
         messages may be sent. When send() is called while a previous deferred/future
         has not yet fired, the send will fail immediately.
 
-        :param message: An instance that implements :class:`autobahn.wamp.interfaces.IMessage`
-        :type message: obj
+        :param message: The WAMP message to send over the transport.
+        :type message: object implementing :class:`autobahn.wamp.interfaces.IMessage`
 
         :returns: obj -- A Deferred/Future
         """
 
+    @public
     @abc.abstractmethod
-    def is_open(self):
+    def isOpen():
         """
         Check if the transport is open for messaging.
 
-        :returns: bool -- ``True``, if the transport is open.
+        :returns: ``True``, if the transport is open.
+        :rtype: bool
         """
 
+    @public
     @abc.abstractmethod
-    def close(self):
+    def close():
         """
         Close the transport regularly. The transport will perform any
         closing handshake if applicable. This should be used for any
         application initiated closing.
         """
 
+    @public
     @abc.abstractmethod
-    def abort(self):
+    def abort():
         """
         Abort the transport abruptly. The transport will be destroyed as
         fast as possible, and without playing nice to the peer. This should
@@ -166,8 +239,9 @@ class ITransport(object):
         detected attacks.
         """
 
+    @public
     @abc.abstractmethod
-    def get_channel_id(self):
+    def get_channel_id():
         """
         Return the unique channel ID of the underlying transport. This is used to
         mitigate credential forwarding man-in-the-middle attacks when running
@@ -211,69 +285,114 @@ class ITransport(object):
         """
 
 
+@public
 @six.add_metaclass(abc.ABCMeta)
 class ITransportHandler(object):
 
+    @public
     @abc.abstractproperty
-    def transport(self):
+    def transport():
         """
         When the transport this handler is attached to is currently open, this property
         can be read from. The property should be considered read-only. When the transport
         is gone, this property is set to None.
         """
 
+    @public
     @abc.abstractmethod
-    def on_open(self, transport):
+    def onOpen(transport):
         """
         Callback fired when transport is open. May run asynchronously. The transport
         is considered running and is_open() would return true, as soon as this callback
         has completed successfully.
 
-        :param transport: An instance that implements :class:`autobahn.wamp.interfaces.ITransport`
-        :type transport: obj
+        :param transport: The WAMP transport.
+        :type transport: object implementing :class:`autobahn.wamp.interfaces.ITransport`
         """
 
+    @public
     @abc.abstractmethod
-    def on_message(self, message):
+    def onMessage(message):
         """
         Callback fired when a WAMP message was received. May run asynchronously. The callback
         should return or fire the returned deferred/future when it's done processing the message.
         In particular, an implementation of this callback must not access the message afterwards.
 
-        :param message: An instance that implements :class:`autobahn.wamp.interfaces.IMessage`
-        :type message: obj
+        :param message: The WAMP message received.
+        :type message: object implementing :class:`autobahn.wamp.interfaces.IMessage`
         """
 
+    @public
     @abc.abstractmethod
-    def on_close(self, was_clean):
+    def onClose(wasClean):
         """
         Callback fired when the transport has been closed.
 
-        :param was_clean: Indicates if the transport has been closed regularly.
-        :type was_clean: bool
+        :param wasClean: Indicates if the transport has been closed regularly.
+        :type wasClean: bool
         """
 
 
+@public
 @six.add_metaclass(abc.ABCMeta)
 class ISession(object):
     """
-    Base interface for WAMP sessions.
+    Interface for WAMP sessions.
     """
 
     @abc.abstractmethod
-    def on_connect(self):
+    def __init__(config=None):
+        """
+
+        :param config: Configuration for session.
+        :type config: instance of :class:`autobahn.wamp.types.ComponentConfig`.
+        """
+
+    @public
+    @abc.abstractmethod
+    def onUserError(fail, msg):
+        """
+        This is called when we try to fire a callback, but get an
+        exception from user code -- for example, a registered publish
+        callback or a registered method. By default, this prints the
+        current stack-trace and then error-message to stdout.
+
+        ApplicationSession-derived objects may override this to
+        provide logging if they prefer. The Twisted implemention does
+        this. (See :class:`autobahn.twisted.wamp.ApplicationSession`)
+
+        :param fail: The failure that occurred.
+        :type fail: instance implementing txaio.IFailedFuture
+
+        :param msg: an informative message from the library. It is
+            suggested you log this immediately after the exception.
+        :type msg: str
+        """
+
+    @public
+    @abc.abstractmethod
+    def onConnect():
         """
         Callback fired when the transport this session will run over has been established.
         """
 
+    @public
     @abc.abstractmethod
-    def join(self, realm):
+    def join(realm,
+             authmethods=None,
+             authid=None,
+             authrole=None,
+             authextra=None,
+             resumable=None,
+             resume_session=None,
+             resume_token=None):
         """
         Attach the session to the given realm. A session is open as soon as it is attached to a realm.
         """
 
+    @public
     @abc.abstractmethod
-    def on_challenge(self, challenge):
+    def onChallenge(challenge):
         """
         Callback fired when the peer demands authentication.
 
@@ -283,8 +402,9 @@ class ISession(object):
         :type challenge: Instance of :class:`autobahn.wamp.types.Challenge`.
         """
 
+    @public
     @abc.abstractmethod
-    def on_join(self, details):
+    def onJoin(details):
         """
         Callback fired when WAMP session has been established.
 
@@ -294,8 +414,9 @@ class ISession(object):
         :type details: Instance of :class:`autobahn.wamp.types.SessionDetails`.
         """
 
+    @public
     @abc.abstractmethod
-    def leave(self, reason=None, message=None):
+    def leave(reason=None, message=None):
         """
         Actively close this WAMP session.
 
@@ -310,8 +431,9 @@ class ISession(object):
         :return: may return a Future/Deferred that fires when we've disconnected
         """
 
+    @public
     @abc.abstractmethod
-    def on_leave(self, details):
+    def onLeave(details):
         """
         Callback fired when WAMP session has is closed
 
@@ -319,52 +441,60 @@ class ISession(object):
         :type details: Instance of :class:`autobahn.wamp.types.CloseDetails`.
         """
 
+    @public
     @abc.abstractmethod
-    def disconnect(self):
+    def disconnect():
         """
         Close the underlying transport.
         """
 
+    @public
     @abc.abstractmethod
-    def is_connected(self):
-        """
-        Check if the underlying transport is connected.
-        """
-
-    @abc.abstractmethod
-    def is_attached(self):
-        """
-        Check if the session has currently joined a realm.
-        """
-
-    @abc.abstractmethod
-    def on_disconnect(self):
+    def onDisconnect():
         """
         Callback fired when underlying transport has been closed.
         """
 
-
-@six.add_metaclass(abc.ABCMeta)
-class IApplicationSession(ISession):
-    """
-    Interface for WAMP client peers implementing the four different
-    WAMP roles (caller, callee, publisher, subscriber).
-    """
-
+    @public
     @abc.abstractmethod
-    def define(self, exception, error=None):
+    def is_connected():
+        """
+        Check if the underlying transport is connected.
+        """
+
+    @public
+    @abc.abstractmethod
+    def is_attached():
+        """
+        Check if the session has currently joined a realm.
+        """
+
+    @public
+    def set_keyring(keyring):
+        """
+        Set the WAMP-cryptobox keyring to be used.
+
+        :param keyring: The WAMP-cryptosign keyring.
+        :type keyring: instance of :class:`autobahn.wamp.cryptobox.KeyRing`
+        """
+
+    @public
+    @abc.abstractmethod
+    def define(exception, error=None):
         """
         Defines an exception for a WAMP error in the context of this WAMP session.
 
         :param exception: The exception class to define an error mapping for.
         :type exception: A class that derives of ``Exception``.
+
         :param error: The URI (or URI pattern) the exception class should be mapped for.
-                      Iff the ``exception`` class is decorated, this must be ``None``.
+            Iff the ``exception`` class is decorated, this must be ``None``.
         :type error: str
         """
 
+    @public
     @abc.abstractmethod
-    def call(self, procedure, *args, **kwargs):
+    def call(procedure, *args, **kwargs):
         """
         Call a remote procedure.
 
@@ -388,8 +518,10 @@ class IApplicationSession(ISession):
 
         :param procedure: The URI of the remote procedure to be called, e.g. ``u"com.myapp.hello"``.
         :type procedure: unicode
+
         :param args: Any positional arguments for the call.
         :type args: list
+
         :param kwargs: Any keyword arguments for the call.
         :type kwargs: dict
 
@@ -397,8 +529,9 @@ class IApplicationSession(ISession):
         :rtype: instance of :tx:`twisted.internet.defer.Deferred` / :py:class:`asyncio.Future`
         """
 
+    @public
     @abc.abstractmethod
-    def register(self, endpoint, procedure=None, options=None):
+    def register(endpoint, procedure=None, options=None):
         """
         Register a procedure for remote calling.
 
@@ -419,10 +552,12 @@ class IApplicationSession(ISession):
 
         :param endpoint: The endpoint called under the procedure.
         :type endpoint: callable or object
+
         :param procedure: When ``endpoint`` is a callable, the URI (or URI pattern)
            of the procedure to register for. When ``endpoint`` is an object,
            the argument is ignored (and should be ``None``).
         :type procedure: unicode
+
         :param options: Options for registering.
         :type options: instance of :class:`autobahn.wamp.types.RegisterOptions`.
 
@@ -430,8 +565,9 @@ class IApplicationSession(ISession):
         :rtype: instance(s) of :tx:`twisted.internet.defer.Deferred` / :py:class:`asyncio.Future`
         """
 
+    @public
     @abc.abstractmethod
-    def publish(self, topic, *args, **kwargs):
+    def publish(topic, *args, **kwargs):
         """
         Publish an event to a topic.
 
@@ -455,8 +591,10 @@ class IApplicationSession(ISession):
 
         :param topic: The URI of the topic to publish to, e.g. ``u"com.myapp.mytopic1"``.
         :type topic: unicode
+
         :param args: Arbitrary application payload for the event (positional arguments).
         :type args: list
+
         :param kwargs: Arbitrary application payload for the event (keyword arguments).
         :type kwargs: dict
 
@@ -464,8 +602,9 @@ class IApplicationSession(ISession):
         :rtype: ``None`` or instance of :tx:`twisted.internet.defer.Deferred` / :py:class:`asyncio.Future`
         """
 
+    @public
     @abc.abstractmethod
-    def subscribe(self, handler, topic=None, options=None):
+    def subscribe(handler, topic=None, options=None):
         """
         Subscribe to a topic for receiving events.
 
@@ -486,10 +625,12 @@ class IApplicationSession(ISession):
 
         :param handler: The event handler to receive events.
         :type handler: callable or object
+
         :param topic: When ``handler`` is a callable, the URI (or URI pattern)
            of the topic to subscribe to. When ``handler`` is an object, this
            value is ignored (and should be ``None``).
         :type topic: unicode
+
         :param options: Options for subscribing.
         :type options: An instance of :class:`autobahn.wamp.types.SubscribeOptions`.
 
